namespace MangaReader.Core.Sources.MangaDex.Api;

public interface IMangaDexClient
{
    Task<MangaDexResponse?> SearchMangaByTitleAsync(string title, CancellationToken cancellationToken);
    Task<MangaDexResponse?> SearchMangaByAuthorAsync(string author, CancellationToken cancellationToken);
    Task<MangaDexResponse?> SearchMangaByGroupAsync(string group, CancellationToken cancellationToken);
    Task<MangaDexResponse?> GetMangaAsync(Guid mangaGuid, CancellationToken cancellationToken);
    Task<MangaDexResponse?> GetFeedAsync(Guid mangaGuid, CancellationToken cancellationToken);
    Task<MangaDexChapterResponse?> GetChapterAsync(Guid chapterGuid, CancellationToken cancellationToken);
    Task<MangaDexResponse?> GetCoverArtAsync(Guid mangaGuid, CancellationToken cancellationToken);
    Task<MangaDexResponse?> GetCoverArtAsync(Guid[] mangaGuid, CancellationToken cancellationToken);
}